@@ -11,4 +11,5 @@ fi
 COUNT=$(cat /proc/cpuinfo | grep 'model name' | sed -e 's/.*: //' | wc -l)
 echo "number of detected CPUs =" $COUNT
 
-cppcheck -j $COUNT --force --inline-suppr . 2>errors.txt
+#cppcheck -j $COUNT --force --inline-suppr --enable=warning . 2>errors.txt
+cppcheck -j $COUNT --force --enable=style . 2>errors.txt
\ No newline at end of file
