@@ -20,18 +20,26 @@
  ***************************************************************************/
 
 #include <QtGui>
+#include <QSettings>
+#include <QRadioButton>
+
 
 #include "../main/includes/httraqt.h"
 #include "includes/OptionsDialog.h"
 
 
-OptionsDialog::OptionsDialog(QWidget* parent, Qt::WFlags fl)
+int OptionsDialog::headers[OPTION_SITES] = {_LINKS, _BUILD, _SCANRULEZ, _LIMITS, _PROXY, _EXPERTS, _MIMETYPES, _FLOWCONTROL,
+                                            _SPIDER, _BROWSERID, _LOGINDEX
+                                           };
+
+
+OptionsDialog::OptionsDialog(QWidget* parent, Qt::WindowFlags fl)
     : QDialog(parent, fl)
 {
     setupUi(this);
 
-    this->parent = (HTTraQt*)parent;
-    // qDebug() << "init OptionsDialog";
+    this->parent = static_cast<HTTraQt*>(parent);
+
     _tabTextInfos.clear();
 
     QToolButton* b[] = {buttonLinks, buttonBuild, buttonRulez, buttonLimits, buttonProxy,
@@ -39,73 +47,86 @@ OptionsDialog::OptionsDialog(QWidget* parent, Qt::WFlags fl)
                        };
     memcpy(buttons, b, OPTION_SITES * sizeof(QToolButton*));
 
-    int h[] = {_LINKS, _BUILD, _SCANRULEZ, _LIMITS, _PROXY, _EXPERTS, _MIMETYPES, _FLOWCONTROL,
-               _SPIDER, _BROWSERID, _LOGINDEX
-              };
-    memcpy(headers, h, OPTION_SITES * sizeof(int));
-
     for (int i = 0; i < OPTION_SITES; i++) {
-        //         headers[i] = buttons[i]->text();
-        buttons[i]->setText(translateString(headers[i]));
+        buttons[i]->setText(translate(headers[i]));
     }
 
     pBuild = new optionsBuild(this);
-    gridLayoutM->addWidget((QWidget*)pBuild);
+    gridWidget->addWidget((QWidget*)pBuild);
 
     pBrowser = new optionsBrowser(this);
-    gridLayoutM->addWidget((QWidget*)pBrowser);
+    gridWidget->addWidget((QWidget*)pBrowser);
 
     pFlow = new optionsFlow(this);
-    gridLayoutM->addWidget((QWidget*)pFlow);
+    gridWidget->addWidget((QWidget*)pFlow);
 
     pLimits = new optionsLimits(this);
-    gridLayoutM->addWidget((QWidget*)pLimits);
+    gridWidget->addWidget((QWidget*)pLimits);
 
     pLinks = new optionsLinks(this);
-    gridLayoutM->addWidget((QWidget*)pLinks);
+    gridWidget->addWidget((QWidget*)pLinks);
 
     pProxy = new optionsProxy(this);
-    gridLayoutM->addWidget((QWidget*)pProxy);
+    gridWidget->addWidget((QWidget*)pProxy);
 
     pRulez = new optionsRulez(this);
-    gridLayoutM->addWidget((QWidget*)pRulez);
+    gridWidget->addWidget((QWidget*)pRulez);
 
     pExperts = new optionsExperts(this);
-    gridLayoutM->addWidget((QWidget*)pExperts);
+    gridWidget->addWidget((QWidget*)pExperts);
 
     pLog = new optionsLog(this);
-    gridLayoutM->addWidget((QWidget*)pLog);
+    gridWidget->addWidget((QWidget*)pLog);
 
     pSpider = new optionsSpider(this);
-    gridLayoutM->addWidget((QWidget*)pSpider);
+    gridWidget->addWidget((QWidget*)pSpider);
 
     pMime = new optionsMime(this);
-    gridLayoutM->addWidget((QWidget*)pMime);
+    gridWidget->addWidget((QWidget*)pMime);
 
     initOptions();
 
-    setToGUI();
-
     setFontForWidgets();
 
+    QSettings s(QSettings::UserScope, "KarboSoft", "HTTraQt");
+    QPoint pos = s.value("posSettings", QPoint(200, 200)).toPoint();
+    QSize size = s.value("sizeSettings", QSize(830, 480)).toSize();
+    resize(size);
+    move(pos);
+
+    //      connect(gridWidget, SIGNAL(resizeEvent()), this, SLOT(onOk()));
+
     connect(buttonOk, SIGNAL(clicked()), this, SLOT(onOk()));
     connect(buttonCancel, SIGNAL(clicked()), this, SLOT(reject()));
     connect(pushDefault, SIGNAL(clicked()), this, SLOT(onSaveDefOptions()));
     connect(buttonHelp, SIGNAL(clicked()), this, SLOT(onHelp()));
 
     for (int i = 0; i < OPTION_SITES; i++) {
-        connect(buttons[i], SIGNAL(toggled(bool)), this, SLOT(switchPages()));
+        connect(buttons[i], SIGNAL(clicked(bool)), this, SLOT(switchPages()));
     }
 
     QWidget* p[] = {pLinks, pBuild, pRulez, pLimits, pProxy, pExperts, pMime, pFlow, pSpider, pBrowser, pLog};
     memcpy(pages, p, OPTION_SITES * sizeof(QWidget*));
 
-    buttons[0]->setChecked(true);
+    for (int i = 0; i < OPTION_SITES; i++) {
+        pages[i]->adjustSize();
+    }
+
+    update();
+
+    setToGUI();
+
+    buttons[0]->click();
 }
 
 
 OptionsDialog::~OptionsDialog()
 {
+    QSettings s(QSettings::UserScope, "KarboSoft", "HTTraQt");
+    s.setValue("posSettings", pos());
+    s.setValue("sizeSettings", size());
+    s.sync();
+
     delete pBuild;
     delete pBrowser;
     delete pFlow;
@@ -120,6 +141,58 @@ OptionsDialog::~OptionsDialog()
 }
 
 
+void OptionsDialog::resizeEvent(QResizeEvent* se)
+{
+    // QSize s = gridWidget->sizeHint();
+
+    for (int i = 0; i < OPTION_SITES; i++) {
+        pages[i]->adjustSize();
+    }
+
+    QVector<trWidgets>::iterator ivec;
+
+    for (ivec = _tabTextInfos.begin(); ivec != _tabTextInfos.end(); ++ivec) {
+        switch ((*ivec).wtype) {
+            case LABEL: {
+                if((*ivec).gui_ID != -1) {
+                    QString t = autoWrap(translate((*ivec).gui_ID), ((QLabel*)(*ivec).wg)->width() / 10.0); // autowrap
+                    //                     ((QLabel*)(*ivec).wg)->update();
+                    //                     qDebug() << "LABEL" << ((QLabel*)(*ivec).wg)->width();
+                    ((QLabel*)(*ivec).wg)->setText(t);
+                }
+
+                break;
+            }
+
+            case CHECKBOX: {
+                if((*ivec).gui_ID != -1) {
+                    QString t = autoWrap(translate((*ivec).gui_ID), ((QCheckBox*)(*ivec).wg)->width() / 10.0); // autowrap
+                    //                     ((QCheckBox*)(*ivec).wg)->update();
+                    //                     qDebug() << "CHECKBOX" << ((QCheckBox*)(*ivec).wg)->width();
+                    ((QCheckBox*)(*ivec).wg)->setText(t);
+                }
+
+                break;
+            }
+
+            case RADIO: {
+                if((*ivec).gui_ID != -1) {
+                    QString t = autoWrap(translate((*ivec).gui_ID), ((QRadioButton*)(*ivec).wg)->width() / 10.0); // autowrap
+                    //                     ((QRadioButton*)(*ivec).wg)->update();
+                    //                     qDebug() << "RADIO" << ((QRadioButton*)(*ivec).wg)->width();
+                    ((QRadioButton*)(*ivec).wg)->setText(t);
+                }
+
+                break;
+            }
+
+        }
+    }
+
+    QDialog::resizeEvent(se);
+}
+
+
 void OptionsDialog::setFontForWidgets(void)
 {
     QFont f = parent->sysFont;
@@ -146,24 +219,18 @@ void OptionsDialog::setFontForWidgets(void)
 
 void OptionsDialog::initOptions()
 {
-    QVector<options>::iterator iopt;
+    for (QMap<QString, OptStruct>::const_iterator iopt = parent->prOptions.constBegin(); iopt != parent->prOptions.constEnd(); ++iopt) {
+        QString k = iopt.key();
+        trWidgets* w = findId(k);
 
-    for (iopt = parent->prOptions.begin(); iopt != parent->prOptions.end(); ++iopt) {
-        QVector<trWidgets>::iterator ivec;
-        bool found = false;
-
-        for (ivec = _tabTextInfos.begin(); ivec != _tabTextInfos.end(); ++ivec) {
-            if ((*ivec).idString == (*iopt).name) {
-                found = true;
-
-                (*ivec).value = (*iopt).value;
-            }
+        if (w != NULL) {
+            (*w).value = (*iopt).value;
         }
 
 #ifdef MY_DEBUG
 
-        if (found == false) {
-            qDebug() << "set to gui" << (*iopt).name << "not found in gui!";
+        if (w == NULL) {
+            qDebug() << "set to gui" << iopt.key() << "not found in gui!";
         }
 
 #endif
@@ -171,129 +238,178 @@ void OptionsDialog::initOptions()
 }
 
 
+trWidgets *OptionsDialog::findId(const QString &k)
+{
+    QVector<trWidgets>::iterator ivec;
+
+    for (ivec = _tabTextInfos.begin(); ivec != _tabTextInfos.end(); ++ivec) {
+        if ((*ivec).idString == k) {
+            return ivec;
+        }
+    }
+
+    return NULL;
+}
+
+
 void OptionsDialog::getFromGUI()
 {
-    QVector<options>::iterator iopt;
+    for (QMap<QString, OptStruct>::iterator iopt = parent->prOptions.begin(); iopt != parent->prOptions.end(); ++iopt) {
+        QString k = iopt.key();
+        trWidgets* w = findId(k);
 
-    for (iopt = parent->prOptions.begin(); iopt != parent->prOptions.end(); ++iopt) {
-        QVector<trWidgets>::iterator ivec;
-        bool found = false;
+        if (w == NULL) {
+#ifdef MY_DEBUG
+            qDebug() << "get from gui" << iopt.key() << "not found in gui!";
+#endif
+            continue;
+        }
 
-        for (ivec = _tabTextInfos.begin(); ivec != _tabTextInfos.end(); ++ivec) {
-            if ((*ivec).idString == (*iopt).name) {
-                found = true;
+        switch ((*w).wtype) {
+            case CHECKBOX:
+                (*w).value = ((QCheckBox*)(*w).wg)->isChecked() ? 1 : 0;
+                break;
 
-                switch ((*ivec).wtype) {
-                    case CHECKBOX:
-                        (*ivec).value = ((QCheckBox*)(*ivec).wg)->isChecked() ? 1 : 0;
-                        break;
-
-                    case EDITLINE: {
-                        if ((*ivec).opttype == INTEGER) {
-                            if (((QLineEdit*)(*ivec).wg)->text().length() > 0) {
-                                (*ivec).value = ((QLineEdit*)(*ivec).wg)->text().toInt();
-                            }
-                        } else {
-                            (*ivec).value = -1;
-                        }
-
-                        if ((*ivec).opttype == FLOAT) {
-                            if (((QLineEdit*)(*ivec).wg)->text().length() > 0) {
-                                (*ivec).value = ((QLineEdit*)(*ivec).wg)->text().toFloat();
-                            } else {
-                                (*ivec).value = -1;
-                            }
-                        }
-
-                        if ((*ivec).opttype == TEXT) {
-                            (*ivec).value = ((QLineEdit*)(*ivec).wg)->text();
-                        }
-
-                        break;
+            case EDITLINE: {
+                if ((*w).opttype == INTEGER) {
+                    if (((QLineEdit*)(*w).wg)->text().length() > 0) {
+                        (*w).value = ((QLineEdit*)(*w).wg)->text().toInt();
                     }
+                } else {
+                    (*w).value = -1;
+                }
 
-                    case LABEL: {
-                        if ((*ivec).opttype == TEXT) {
-                            (*ivec).value = ((QLabel*)(*ivec).wg)->text();
-                        }
-
-                        break;
-                    }
-
-                    case TEXTEDIT: { // this one only for rulez->scanList
-                        (*ivec).value = ((QTextEdit*)(*ivec).wg)->toPlainText();
-                        break;
-                    }
-
-                    case RADIO: // not exists
-                        break;
-
-                    case GROUPBOX: {
-                        if ((*ivec).opttype == BOOLEAN) {
-                            (*ivec).value = ((QGroupBox*)(*ivec).wg)->isChecked() ? 1 : 0;
-                        }
-
-                        break;
-                    }
-
-                    case COMBOBOX: {
-                        if ((*ivec).opttype == TEXT) {
-                            (*ivec).value = ((QComboBox*)(*ivec).wg)->currentText();
-                        }
-
-                        if ((*ivec).opttype == INTEGER || (*ivec).opttype == FLOAT) {
-                            bool ok;
-                            QString ct = ((QComboBox*)(*ivec).wg)->currentText();
-
-                            if (ct == "" || ct == "-") {
-                                //                                 qDebug() << "get options combo" << (*ivec).idString << (*ivec).value;
-                                if ((*ivec).opttype == INTEGER ) {
-                                    (*ivec).value = -1;
-                                }
-
-                                if ((*ivec).opttype == FLOAT ) {
-                                    (*ivec).value = -1.0;
-                                }
-                            } else {
-                                if ((*ivec).opttype == INTEGER ) {
-                                    int  num = ct.toInt(&ok);
-
-                                    if (ok == true) {
-                                        (*ivec).value = num;
-                                    } else {
-                                        (*ivec).value = ((QComboBox*)(*ivec).wg)->currentIndex();
-                                    }
-                                } else {
-                                    float  num = ct.toFloat(&ok);
-
-                                    if (ok == true) {
-                                        (*ivec).value = num;
-                                    } else {
-                                        (*ivec).value = ((QComboBox*)(*ivec).wg)->currentIndex();
-                                    }
-                                }
-                            }
-                        }
-
-                        break;
+                if ((*w).opttype == FLOAT) {
+                    if (((QLineEdit*)(*w).wg)->text().length() > 0) {
+                        (*w).value = ((QLineEdit*)(*w).wg)->text().toFloat();
+                    } else {
+                        (*w).value = -1;
                     }
                 }
 
-                (*iopt).value = (*ivec).value;
+                if ((*w).opttype == TEXT) {
+                    (*w).value = ((QLineEdit*)(*w).wg)->text();
+                }
+
+                break;
+            }
+
+            case LABEL: {
+                if ((*w).opttype == TEXT) {
+                    (*w).value = ((QLabel*)(*w).wg)->text();
+                }
+
+                break;
+            }
+
+            case TEXTEDIT: { // this one only for rulez->scanList
+                (*w).value = ((QTextEdit*)(*w).wg)->toPlainText();
+                break;
+            }
+
+            case RADIO: // not exists
+                break;
+
+            case GROUPBOX: {
+                if ((*w).opttype == BOOLEAN) {
+                    (*w).value = ((QGroupBox*)(*w).wg)->isChecked() ? 1 : 0;
+                }
+
+                break;
+            }
+
+            case COMBOBOX: {
+                if ((*w).opttype == TEXT) {
+                    (*w).value = ((QComboBox*)(*w).wg)->currentText();
+                }
+
+                if ((*w).opttype == INTEGER || (*w).opttype == FLOAT) {
+                    bool ok;
+                    QString ct = ((QComboBox*)(*w).wg)->currentText();
+
+                    if (ct == "" || ct == "-") {
+                        //                                 qDebug() << "get options combo" << (*w).idString << (*w).value;
+                        if ((*w).opttype == INTEGER ) {
+                            (*w).value = -1;
+                        }
+
+                        if ((*w).opttype == FLOAT ) {
+                            (*w).value = -1.0;
+                        }
+                    } else {
+                        if ((*w).opttype == INTEGER ) {
+                            int  num = ct.toInt(&ok);
+
+                            if (ok == true) {
+                                (*w).value = num;
+                            } else {
+                                (*w).value = ((QComboBox*)(*w).wg)->currentIndex();
+                            }
+                        } else {
+                            float  num = ct.toFloat(&ok);
+
+                            if (ok == true) {
+                                (*w).value = num;
+                            } else {
+                                (*w).value = ((QComboBox*)(*w).wg)->currentIndex();
+                            }
+                        }
+                    }
+                }
+
+                break;
             }
         }
 
-#ifdef MY_DEBUG
-
-        if (found == false) {
-            qDebug() << "get from gui" << (*iopt).name << "not found in gui!";
-        }
-
-#endif
+        (*iopt).value = (*w).value;
     }
 }
 
 
+QString OptionsDialog::autoWrap(const QString &t, const int max)
+{
+    QString res;
+    QString tmp = t;
+
+    //     if (max == NOWRAP ) {
+    //         return tmp;
+    //     }
+
+    tmp.replace("\n", " ");
+    int pos;
+
+    if (t.length() == 0) {
+        return "";
+    }
+
+    if (t.length() < max) {
+        return tmp;
+    }
+
+    pos = tmp.lastIndexOf(" ", max);
+
+    if (pos <= 0 ) {
+        return tmp;
+    }
+
+    while (pos > 0) {
+        //         if (pos > 0){
+        res += tmp.left(pos) + "\n";
+        tmp = tmp.mid(pos + 1);
+        //         }
+        //         res += "\n";
+        pos = tmp.lastIndexOf(" ", max);
+
+        if (pos < 0) {
+            res += tmp;
+            break;
+        }
+    };
+
+    return res;
+}
+
+
 void OptionsDialog::setToGUI()
 {
     QVector<trWidgets>::iterator ivec;
@@ -301,10 +417,11 @@ void OptionsDialog::setToGUI()
     for (ivec = _tabTextInfos.begin(); ivec != _tabTextInfos.end(); ++ivec) {
         switch ((*ivec).wtype) {
             case LABEL: {
-                if ((*ivec).opttype == NOTYPE) {
-                    ((QLabel*)(*ivec).wg)->setText(translateString((*ivec).gui_ID));
+                if((*ivec).gui_ID != -1) {
+                    QString t = autoWrap(translate((*ivec).gui_ID), ((QLabel*)(*ivec).wg)->width() / 10); // autowrap
+                    ((QLabel*)(*ivec).wg)->setText(t);
                 } else {
-                    ((QLabel*)(*ivec).wg)->setText(translateString((*ivec).value.toInt()));
+                    ((QLabel*)(*ivec).wg)->setText(translate((*ivec).value.toInt()));
                 }
 
                 break;
@@ -316,16 +433,20 @@ void OptionsDialog::setToGUI()
             }
 
             case BUTTON: {
-                ((QPushButton*)(*ivec).wg)->setText(translateString((*ivec).gui_ID));
+                if((*ivec).gui_ID != -1) {
+                    ((QPushButton*)(*ivec).wg)->setText(translate((*ivec).gui_ID));
+                }
+
                 break;
             }
 
             case CHECKBOX: {
-                if ((*ivec).opttype == TEXT) {
-                    ((QCheckBox*)(*ivec).wg)->setText(translateString((*ivec).gui_ID));
-                } else {
-                    ((QCheckBox*)(*ivec).wg)->setText(translateString((*ivec).gui_ID));
+                if((*ivec).gui_ID != -1) {
+                    QString t = autoWrap(translate((*ivec).gui_ID), ((QCheckBox*)(*ivec).wg)->width() / 10); // autowrap
+                    ((QCheckBox*)(*ivec).wg)->setText(t);
+                }
 
+                if ((*ivec).opttype != TEXT) {
                     if ((*ivec).value.toInt() == 1) {
                         ((QCheckBox*)(*ivec).wg)->setCheckState(Qt::Checked);
                     } else {
@@ -338,7 +459,9 @@ void OptionsDialog::setToGUI()
             }
 
             case GROUPBOX: {
-                ((QGroupBox*)(*ivec).wg)->setTitle(translateString((*ivec).gui_ID));
+                if((*ivec).gui_ID != -1) {
+                    ((QGroupBox*)(*ivec).wg)->setTitle(translate((*ivec).gui_ID));
+                }
 
                 if ((*ivec).opttype == BOOLEAN) {
                     if ((*ivec).value.toInt() == 1) {
@@ -373,10 +496,6 @@ void OptionsDialog::setToGUI()
                         break;
                     }
 
-                    //                     if ((*ivec).gui_ID != -1) {
-                    //                         ((QComboBox*)(*ivec).wg)->addItem(translateString((*ivec).gui_ID));
-                    //                     }
-
                     if ((*ivec).opttype == INTEGER) {
                         int pos;
 
@@ -431,7 +550,11 @@ void OptionsDialog::setToGUI()
             }
 
             case RADIO: {
-                ((QRadioButton*)(*ivec).wg)->setText(translateString((*ivec).gui_ID));
+                if((*ivec).gui_ID != -1) {
+                    QString t = autoWrap(translate((*ivec).gui_ID), ((QRadioButton*)(*ivec).wg)->width() / 10); // autowrap
+                    ((QRadioButton*)(*ivec).wg)->setText(t);
+                }
+
                 break;
             }
 
@@ -485,36 +608,31 @@ void OptionsDialog::switchPages()
         return;
     }
 
-    pages[n]->show();
-    labelMain->setText(translateString(headers[n]));
-
-
     for (int i = 0; i < OPTION_SITES; i++) {
-        disconnect(buttons[i], SIGNAL(toggled(bool)), this, SLOT(switchPages()));
+        disconnect(buttons[i], SIGNAL(clicked(bool)), this, SLOT(switchPages()));
     }
 
-
     for (int i = 0; i < OPTION_SITES; i++) {
         if (i == n) {
+            labelMain->setText(translate(headers[i]));
+            pages[i]->show();
+            //             pages[i]->adjustSize();
+            buttons[i]->setChecked(true);
             continue;
         }
 
         pages[i]->hide();
-
         buttons[i]->setChecked(false);
     }
 
-    buttons[n]->setChecked(true);
-
     for (int i = 0; i < OPTION_SITES; i++) {
-        connect(buttons[i], SIGNAL(toggled(bool)), this, SLOT(switchPages()));
+        connect(buttons[i], SIGNAL(clicked(bool)), this, SLOT(switchPages()));
     }
 }
 
 
 void OptionsDialog::onResetDefOptions()
 {
-    //     getFromGUI();
     parent->resetDefaultOptions();
     parent->writeSettings(true);
 }
